
import bpy
from bpy.types import Panel


# [ Keymap Register ]
addon_keymaps_list = []
def set_preset_keymap(bl_idname, hotkey_kwargs, *, key_cat='Window', space_type='EMPTY', op_kwargs={}):
    wm = bpy.context.window_manager
    addon_keyconfig = wm.keyconfigs.addon
    addon_keymaps = addon_keyconfig.keymaps
    addon_km = addon_keymaps.get(key_cat)
    if not addon_km:
        addon_km = addon_keymaps.new(name=key_cat, space_type=space_type)

    addon_kmi = addon_km.keymap_items.new(bl_idname, **hotkey_kwargs)
    for key in op_kwargs:
        value = op_kwargs[key]
        setattr(addon_kmi.properties, key, value)
    addon_keymaps_list.append((addon_km,addon_kmi.idname))


def keymap_register():
    set_preset_keymap(
        "armature_picker.dialog_panel",
        hotkey_kwargs={'type': "ONE", 'value': "PRESS",'shift':1},
        key_cat='Window',
        space_type='EMPTY',
    )

    set_preset_keymap(
        'wm.call_menu_pie',
        hotkey_kwargs={'type': "RIGHTMOUSE", 'value': "PRESS",'ctrl':1},
        key_cat='Window',
        space_type='EMPTY',
        op_kwargs={'name': 'SOLOPIE_MT_pie_menu'},
    )

def keymap_unregister():
    for km, kmi_idname in addon_keymaps_list:
        for kmi in km.keymap_items:
            if kmi.idname == kmi_idname:
                km.keymap_items.remove(kmi)
    addon_keymaps_list.clear()



# [ Keymap Panel ]
class ARMPIE_PT_keymap(Panel):
    bl_label = "Keymaps"
    bl_idname = "ARMPIE_PT_keymap"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "ArmPie"
    bl_options = {'DEFAULT_CLOSED','HEADER_LAYOUT_EXPAND'}
    bl_order = -1
    def draw(self, context):
        # keymap
        wm = bpy.context.window_manager
        kc = wm.keyconfigs.user
        kms = kc.keymaps
        km = kms.get('Window')
        kis = km.keymap_items
        layout = self.layout
        col = layout.column(align=True)
        
        # [ Solo Pie Settings ]
        row_title = col.row()
        row_title.alignment = 'CENTER'
        row_title.label(text='Solo Pie')
        box_solo = col.box()
        row = box_solo.row()
        kmi_1 = kis.get('wm.call_menu_pie',None)
        if kmi_1:
            row.label(text=kmi_1.idname)
            row.prop(kmi_1, "type", text="", full_event=True)

        # -- Use active or specific object --
        soloPie = context.scene.solo_pie
        showUseActive = soloPie.get('show_use_active',True)

        col_detail = box_solo.column(align=True)
        act_icon = 'RADIOBUT_ON' if showUseActive else 'RADIOBUT_OFF'
        act_text = "Using Active Object" if showUseActive else "Using Specific Object"
        col_detail.prop(soloPie,'show_use_active',text=act_text,icon=act_icon)

        if not showUseActive :
            row = col_detail.row()
            row.enabled = not showUseActive
            row.prop(soloPie,'armature',text='',icon='OUTLINER_OB_ARMATURE')
        
        col.separator(factor=4.0,type='LINE')

        # [ Armature Picker Settings ]
        row_title = col.row()
        row_title.alignment = 'CENTER'
        row_title.label(text='Armature Picker')
        box_solo = col.box()
        row = box_solo.row()
        kmi_2 = kis.get("armature_picker.dialog_panel",None)
        if kmi_2:
            row.label(text=kmi_2.idname)
            row.prop(kmi_2, "type", text="", full_event=True)
        
        col.separator(factor=4.0)
# --- Register ---
def register():
    keymap_register()
    bpy.utils.register_class(ARMPIE_PT_keymap)

def unregister():
    bpy.utils.unregister_class(ARMPIE_PT_keymap)
    keymap_unregister()